System:
Provide a floating-point representation of the sentiment of the following customer product review that is rounded to five decimal places. The scale ranges from -1.0 (negative) to 1.0 (positive) where 0.0 represents neutral sentiment. Only return the sentiment score value and nothing else. You may only return a single numeric value.

User:
Conversation: {{transcribed_and_diarized}}
